@@ -1,7 +1,11 @@
 from __future__ import annotations
 
+import datetime
+import hashlib
+import json
 import os
 import shutil
+import sys
 import textwrap
 from argparse import Namespace
 from collections import defaultdict
@@ -9,18 +13,18 @@ from itertools import chain
 from pathlib import Path
 from typing import Iterable, Mapping, Sequence, cast
 
-import atoml
 import click
+import tomlkit
 from resolvelib.reporters import BaseReporter
 from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep, Resolver
 
-from pdm import termui
+from pdm import signals, termui
 from pdm.cli.utils import (
     check_project_file,
     find_importable_files,
     format_lockfile,
     format_resolution_impossible,
-    frozen_requirement_from_dist,
+    get_dist_location,
     merge_dictionary,
     save_version_specifiers,
     set_env_in_reg,
@@ -29,7 +33,6 @@ from pdm.cli.utils import (
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
-from pdm.installers.manager import format_dist
 from pdm.models.candidates import Candidate
 from pdm.models.python import PythonInfo
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
@@ -47,9 +50,13 @@ def do_lock(
     tracked_names: Iterable[str] | None = None,
     requirements: list[Requirement] | None = None,
     dry_run: bool = False,
+    refresh: bool = False,
 ) -> dict[str, Candidate]:
     """Performs the locking process and update lockfile."""
     check_project_file(project)
+    if refresh:
+        project.write_lockfile(project.lockfile)
+        return project.lockfile
     # TODO: multiple dependency definitions for the same package.
     provider = project.get_provider(strategy, tracked_names)
     if not requirements:
@@ -64,8 +71,9 @@ def do_lock(
         with ui.open_spinner(title="Resolving dependencies", spinner="dots") as spin:
             reporter = project.get_reporter(requirements, tracked_names, spin)
             resolver: Resolver = project.core.resolver_class(provider, reporter)
+            signals.pre_lock.send(project, requirements=requirements, dry_run=dry_run)
             try:
-                mapping, dependencies, summaries = resolve(
+                mapping, dependencies = resolve(
                     resolver,
                     requirements,
                     project.environment.python_requires,
@@ -84,10 +92,11 @@ def do_lock(
             except ResolutionImpossible as err:
                 spin.fail(f"{termui.Emoji.LOCK} Lock failed")
                 ui.echo(format_resolution_impossible(err), err=True)
-                raise
+                raise ResolutionImpossible("Unable to find a resolution") from None
             else:
-                data = format_lockfile(mapping, dependencies, summaries)
+                data = format_lockfile(mapping, dependencies)
                 spin.succeed(f"{termui.Emoji.LOCK} Lock successful")
+            signals.post_lock.send(project, resolution=mapping, dry_run=dry_run)
 
     project.write_lockfile(data, write=not dry_run)
 
@@ -118,6 +127,30 @@ def resolve_candidates_from_lockfile(
     return mapping
 
 
+def check_lockfile(project: Project, raise_not_exist: bool = True) -> str | None:
+    """Check if the lock file exists and is up to date. Return the update strategy."""
+    if not project.lockfile_file.exists():
+        if raise_not_exist:
+            raise ProjectError("Lock file does not exist, nothing to install")
+        project.core.ui.echo("Lock file does not exist", fg="yellow", err=True)
+        return "all"
+    elif not project.is_lockfile_compatible():
+        project.core.ui.echo(
+            "Lock file version is not compatible with PDM, installation may fail",
+            fg="yellow",
+            err=True,
+        )
+        return "all"
+    elif not project.is_lockfile_hash_match():
+        project.core.ui.echo(
+            "Lock file hash doesn't match pyproject.toml, packages may be outdated",
+            fg="yellow",
+            err=True,
+        )
+        return "reuse"
+    return None
+
+
 def do_sync(
     project: Project,
     *,
@@ -130,22 +163,10 @@ def do_sync(
     tracked_names: Sequence[str] | None = None,
     no_editable: bool = False,
     no_self: bool = False,
+    reinstall: bool = False,
 ) -> None:
     """Synchronize project"""
     if requirements is None:
-        if not project.lockfile_file.exists():
-            raise ProjectError("Lock file does not exist, nothing to sync")
-        elif not project.is_lockfile_compatible():
-            project.core.ui.echo(
-                "Lock file version is not compatible with PDM, "
-                "install may fail, please regenerate the pdm.lock",
-                err=True,
-            )
-        elif not project.is_lockfile_hash_match():
-            project.core.ui.echo(
-                "Lock file hash doesn't match pyproject.toml, packages may be outdated",
-                err=True,
-            )
         groups = translate_groups(project, default, dev, groups or ())
         requirements = []
         for group in groups:
@@ -155,7 +176,6 @@ def do_sync(
         candidates = {
             name: c for name, c in candidates.items() if name in tracked_names
         }
-
     handler = project.core.synchronizer_class(
         candidates,
         project.environment,
@@ -164,8 +184,11 @@ def do_sync(
         no_editable=no_editable,
         install_self=not no_self and "default" in groups and bool(project.meta.name),
         use_install_cache=project.config["feature.install_cache"],
+        reinstall=reinstall,
     )
+    signals.pre_install.send(project, candidates=candidates, dry_run=dry_run)
     handler.synchronize()
+    signals.post_install.send(project, candidates=candidates, dry_run=dry_run)
 
 
 def do_add(
@@ -180,6 +203,8 @@ def do_add(
     unconstrained: bool = False,
     no_editable: bool = False,
     no_self: bool = False,
+    dry_run: bool = False,
+    prerelease: bool = False,
 ) -> None:
     """Add packages and install"""
     check_project_file(project)
@@ -193,6 +218,7 @@ def do_add(
         parse_requirement(line) for line in packages
     ]:
         key = r.identify()
+        r.prerelease = prerelease
         tracked_names.add(key)
         requirements[key] = r
     project.core.ui.echo(
@@ -206,12 +232,13 @@ def do_add(
             req.specifier = get_specifier("")
     group_deps.update(requirements)
     reqs = [r for deps in all_dependencies.values() for r in deps.values()]
-    resolved = do_lock(project, strategy, tracked_names, reqs)
+    resolved = do_lock(project, strategy, tracked_names, reqs, dry_run=dry_run)
 
     # Update dependency specifiers and lockfile hash.
     deps_to_update = group_deps if unconstrained else requirements
     save_version_specifiers({group: deps_to_update}, resolved, save)
-    project.add_dependencies(deps_to_update, group, dev)
+    if not dry_run:
+        project.add_dependencies(deps_to_update, group, dev)
     lockfile = project.lockfile
     project.write_lockfile(lockfile, False)
 
@@ -222,6 +249,8 @@ def do_add(
             default=False,
             no_editable=no_editable,
             no_self=no_self,
+            requirements=list(group_deps.values()),
+            dry_run=dry_run,
         )
 
 
@@ -237,8 +266,10 @@ def do_update(
     top: bool = False,
     dry_run: bool = False,
     packages: Sequence[str] = (),
+    sync: bool = True,
     no_editable: bool = False,
     no_self: bool = False,
+    prerelease: bool = False,
 ) -> None:
     """Update specified packages or all packages"""
     check_project_file(project)
@@ -251,6 +282,8 @@ def do_update(
     updated_deps: dict[str, dict[str, Requirement]] = defaultdict(dict)
     install_dev = True if dev is None else dev
     if not packages:
+        if prerelease:
+            raise PdmUsageError("--prerelease must be used with packages given")
         groups = translate_groups(project, default, install_dev, groups or ())
         for group in groups:
             updated_deps[group] = all_dependencies[group]
@@ -272,6 +305,7 @@ def do_update(
                         termui.green(name, bold=True), group, "dev-" if dev else ""
                     )
                 )
+            dependencies[matched_name].prerelease = prerelease
             updated_deps[group][matched_name] = dependencies[matched_name]
         project.core.ui.echo(
             "Updating packages: {}.".format(
@@ -293,18 +327,21 @@ def do_update(
         reqs,
         dry_run=dry_run,
     )
-    do_sync(
-        project,
-        groups=groups,
-        dev=install_dev,
-        default=default,
-        clean=False,
-        dry_run=dry_run,
-        requirements=[r for deps in updated_deps.values() for r in deps.values()],
-        tracked_names=list(chain.from_iterable(updated_deps.values())) if top else None,
-        no_editable=no_editable,
-        no_self=no_self,
-    )
+    if sync:
+        do_sync(
+            project,
+            groups=groups,
+            dev=install_dev,
+            default=default,
+            clean=False,
+            dry_run=dry_run,
+            requirements=[r for deps in updated_deps.values() for r in deps.values()],
+            tracked_names=list(chain.from_iterable(updated_deps.values()))
+            if top
+            else None,
+            no_editable=no_editable,
+            no_self=no_self,
+        )
     if unconstrained and not dry_run:
         # Need to update version constraints
         save_version_specifiers(updated_deps, resolved, save)
@@ -322,6 +359,7 @@ def do_remove(
     packages: Sequence[str] = (),
     no_editable: bool = False,
     no_self: bool = False,
+    dry_run: bool = False,
 ) -> None:
     """Remove packages from working set and pyproject.toml"""
     check_project_file(project)
@@ -351,8 +389,9 @@ def do_remove(
         for i in matched_indexes:
             del deps[i]
 
-    project.write_pyproject()
-    do_lock(project, "reuse")
+    if not dry_run:
+        project.write_pyproject()
+    do_lock(project, "reuse", dry_run=dry_run)
     if sync:
         do_sync(
             project,
@@ -361,6 +400,7 @@ def do_remove(
             clean=True,
             no_editable=no_editable,
             no_self=no_self,
+            dry_run=dry_run,
         )
 
 
@@ -389,19 +429,22 @@ def do_list(
         if json:
             raise PdmUsageError("--json must be used with --graph")
         if freeze:
-            reqs = [
-                frozen_requirement_from_dist(dist)
-                for dist in sorted(
-                    working_set.values(), key=lambda d: d.metadata["Name"]
-                )
-            ]
-            project.core.ui.echo("".join(reqs))
+            reqs = sorted(
+                (
+                    Requirement.from_dist(dist).as_line()
+                    for dist in sorted(
+                        working_set.values(), key=lambda d: d.metadata["Name"]
+                    )
+                ),
+                key=lambda x: x.lower(),
+            )
+            project.core.ui.echo("\n".join(reqs))
             return
         rows = [
-            (termui.green(k, bold=True), format_dist(v))
+            (termui.green(k, bold=True), termui.yellow(v.version), get_dist_location(v))
             for k, v in sorted(working_set.items())
         ]
-        project.core.ui.display_columns(rows, ["Package", "Version"])
+        project.core.ui.display_columns(rows, ["Package", "Version", "Location"])
 
 
 def do_build(
@@ -458,19 +501,18 @@ def do_init(
             "license": make_inline_table({"text": license}),
             "urls": {"homepage": ""},
             "dependencies": make_array([], True),
-            "requires-python": python_requires,
-            "dynamic": ["classifiers"],
+            "requires-python": "" if python_requires == "*" else python_requires,
         },
         "build-system": {"requires": ["pdm-pep517"], "build-backend": "pdm.pep517.api"},
     }
-    if python_requires and python_requires != "*":
-        get_specifier(python_requires)
+    get_specifier(python_requires)
     if not project.pyproject:
         project._pyproject = data
     else:
         project._pyproject["project"] = data["project"]  # type: ignore
         project._pyproject["build-system"] = data["build-system"]  # type: ignore
     project.write_pyproject()
+    signals.post_init.send(project)
 
 
 def do_use(project: Project, python: str = "", first: bool = False) -> None:
@@ -484,11 +526,18 @@ def do_use(project: Project, python: str = "", first: bool = False) -> None:
     if python:
         python = python.strip()
 
-    found_interpreters = list(
-        dict.fromkeys(filter(version_matcher, project.find_interpreters(python)))
-    )
+    found_interpreters = list(dict.fromkeys(project.find_interpreters(python)))
+    matching_interperters = list(filter(version_matcher, found_interpreters))
     if not found_interpreters:
         raise NoPythonVersion("Python interpreter is not found on the system.")
+    if not matching_interperters:
+        project.core.ui.echo("Interpreters found but not matching:", err=True)
+        for py in found_interpreters:
+            project.core.ui.echo(f"  - {py.executable} ({py.identifier})", err=True)
+        raise NoPythonVersion(
+            "No python is found meeting the requirement "
+            f"{termui.green('python' + str(project.python_requires))}"
+        )
     if first or len(found_interpreters) == 1:
         selected_python = found_interpreters[0]
     else:
@@ -505,22 +554,21 @@ def do_use(project: Project, python: str = "", first: bool = False) -> None:
         )
         selected_python = found_interpreters[int(selection)]
 
-    old_path = project.python.executable if "python.path" in project.config else None
-    new_path = selected_python.executable
+    old_python = project.python if "python.path" in project.config else None
     project.core.ui.echo(
         "Using Python interpreter: {} ({})".format(
-            termui.green(str(new_path)),
+            termui.green(str(selected_python.executable)),
             selected_python.identifier,
         )
     )
     project.python = selected_python
     if (
-        old_path
-        and Path(old_path) != Path(new_path)
+        old_python
+        and Path(old_python.path) != Path(selected_python.path)
         and not project.environment.is_global
     ):
         project.core.ui.echo(termui.cyan("Updating executable scripts..."))
-        project.environment.update_shebangs(old_path, new_path)
+        project.environment.update_shebangs(selected_python.executable)
 
 
 def do_import(
@@ -550,18 +598,18 @@ def do_import(
     if options is None:
         options = Namespace(dev=False, group=None)
     project_data, settings = FORMATS[key].convert(project, filename, options)
-    pyproject = project.pyproject or atoml.document()
+    pyproject = project.pyproject or tomlkit.document()
 
     if "tool" not in pyproject or "pdm" not in pyproject["tool"]:  # type: ignore
-        pyproject.setdefault("tool", {})["pdm"] = atoml.table()
+        pyproject.setdefault("tool", {})["pdm"] = tomlkit.table()
 
     if "project" not in pyproject:
-        pyproject.add("project", atoml.table())  # type: ignore
+        pyproject.add("project", tomlkit.table())  # type: ignore
         pyproject["project"].add(  # type: ignore
-            atoml.comment("PEP 621 project metadata")
+            tomlkit.comment("PEP 621 project metadata")
         )
         pyproject["project"].add(  # type: ignore
-            atoml.comment("See https://www.python.org/dev/peps/pep-0621/")
+            tomlkit.comment("See https://www.python.org/dev/peps/pep-0621/")
         )
 
     merge_dictionary(pyproject["project"], project_data)  # type: ignore
@@ -688,7 +736,7 @@ def migrate_pyproject(project: Project) -> None:
             updated_fields.append("dev-dependencies")
         if updated_fields:
             if "tool" not in pyproject or "pdm" not in pyproject["tool"]:
-                pyproject.setdefault("tool", {})["pdm"] = atoml.table()
+                pyproject.setdefault("tool", {})["pdm"] = tomlkit.table()
             pyproject["tool"]["pdm"].update(settings)
             project.pyproject = pyproject
             project.write_pyproject()
@@ -718,3 +766,66 @@ def migrate_pyproject(project: Project) -> None:
         ),
         err=True,
     )
+
+
+def get_latest_version(project: Project) -> str:
+    """Get the latest version of PDM from PyPI, cache for 7 days"""
+    from pdm.utils import get_finder
+
+    cache_key = hashlib.sha224(sys.executable.encode()).hexdigest()
+    cache_file = project.cache("self-check") / cache_key
+    if cache_file.exists():
+        state = json.loads(cache_file.read_text())
+    else:
+        state = {}
+    current_time = datetime.datetime.utcnow().timestamp()
+    if (
+        state.get("last-check")
+        and current_time - state["last-check"] < 60 * 60 * 24 * 7
+    ):
+        return cast(str, state["latest-version"])
+    candidate = get_finder([]).find_best_candidate("pdm")
+    latest_version = str(candidate.best_candidate.version)
+    state.update({"latest-version": latest_version, "last-check": current_time})
+    cache_file.write_text(json.dumps(state))
+    return latest_version
+
+
+def check_update(project: Project) -> None:
+    """Check if there is a new version of PDM available"""
+    import sys
+    from shlex import quote
+
+    from pip._vendor.packaging.version import parse as parse_version
+
+    from pdm.cli.utils import (
+        is_homebrew_installation,
+        is_pipx_installation,
+        is_scoop_installation,
+    )
+
+    this_version = parse_version(project.core.version)
+    latest_version = parse_version(get_latest_version(project))
+    if this_version >= latest_version:
+        return
+    if is_pipx_installation():  # pragma: no cover
+        install_command = "$ pipx upgrade pdm"
+    elif is_scoop_installation():  # pragma: no cover
+        install_command = "$ scoop update pdm"
+    elif is_homebrew_installation():  # pragma: no cover
+        install_command = "$ brew upgrade pdm"
+    else:
+        install_command = f"$ {quote(sys.executable)} -m pip install -U pdm"
+
+    disable_command = "$ pdm config check_update false"
+
+    message = [
+        termui.blue(f"\nPDM {termui.cyan(str(this_version))}"),
+        termui.blue(f" is installed, while {termui.cyan(str(latest_version))}"),
+        termui.blue(" is available.\n"),
+        termui.blue(f"Please run {termui.green(install_command, bold=True)}"),
+        termui.blue(" to upgrade.\n"),
+        termui.blue(f"Run {termui.green(disable_command, bold=True)}"),
+        termui.blue(" to disable the check."),
+    ]
+    project.core.ui.echo("".join(message), err=True)
