@@ -12,7 +12,9 @@ from pdm.builders import EditableBuilder, WheelBuilder
 from pdm.exceptions import BuildError, CandidateNotFound
 from pdm.models import pip_shims
 from pdm.models.requirements import (
+    FileRequirement,
     Requirement,
+    VcsRequirement,
     _egg_info_re,
     filter_requirements_with_extras,
     parse_metadata_from_source,
@@ -24,8 +26,10 @@ from pdm.utils import (
     create_tracked_tempdir,
     expand_env_vars_in_auth,
     get_rev_from_url,
+    get_venv_like_prefix,
     path_replace,
     populate_link,
+    url_without_fragments,
 )
 
 if sys.version_info >= (3, 8):
@@ -62,6 +66,11 @@ def _get_wheel_metadata_from_wheel(whl_file: str, metadata_directory: str) -> st
     return os.path.join(metadata_directory, dist_info[0].split("/")[0])
 
 
+def _filter_none(data: dict[str, Any]) -> dict[str, Any]:
+    """Return a new dict without None values"""
+    return {k: v for k, v in data.items() if v is not None}
+
+
 class Candidate:
     """A concrete candidate that can be downloaded and installed.
     A candidate comes from the PyPI index of a package, or from the requirement itself
@@ -91,6 +100,7 @@ class Candidate:
         if link is None and self.req:
             link = self.ireq.link
         self.link = link
+        self.summary = ""
         self.source_dir: str | None = None
         self.hashes: dict[str, str] | None = None
         self._requires_python: str | None = None
@@ -149,6 +159,58 @@ class Candidate:
             cast(str, self.ireq.source_dir)
         )
 
+    def direct_url(self) -> dict[str, Any] | None:
+        """PEP 610 direct_url.json data"""
+        req = self.req
+        if isinstance(req, VcsRequirement):
+            if req.editable:
+                assert self.ireq.source_dir
+                return _filter_none(
+                    {
+                        "url": pip_shims.path_to_url(self.ireq.source_dir),
+                        "dir_info": {"editable": True},
+                        "subdirectory": req.subdirectory,
+                    }
+                )
+            return _filter_none(
+                {
+                    "url": url_without_fragments(req.repo),
+                    "vcs_info": _filter_none(
+                        {
+                            "vcs": req.vcs,
+                            "requested_revision": req.ref,
+                            "commit_id": self.revision,
+                        }
+                    ),
+                    "subdirectory": req.subdirectory,
+                }
+            )
+        elif isinstance(req, FileRequirement):
+            if req.is_local_dir:
+                return _filter_none(
+                    {
+                        "url": url_without_fragments(req.url),
+                        "dir_info": _filter_none({"editable": req.editable or None}),
+                        "subdirectory": req.subdirectory,
+                    }
+                )
+            with self.environment.get_finder() as finder:
+                hash_cache = self.environment.project.make_hash_cache()
+                hash_cache.session = finder.session  # type: ignore
+                return _filter_none(
+                    {
+                        "url": url_without_fragments(req.url),
+                        "archive_info": {
+                            "hash": hash_cache.get_hash(
+                                pip_shims.Link(req.url)
+                            ).replace(":", "=")
+                        },
+                        "subdirectory": req.subdirectory,
+                    }
+                )
+        else:
+            return None
+
     def prepare(self, allow_all: bool = False) -> None:
         """Fetch the link of the candidate and unpack to local if necessary.
 
@@ -226,7 +288,6 @@ class Candidate:
             self.req.name = self.name
         if not self.version:
             self.version = result.version  # type: ignore
-        self.requires_python = result.metadata.get("Requires-Python")
         return result
 
     def build(self) -> str:
@@ -239,7 +300,8 @@ class Candidate:
             self.wheel = cached.file_path
             return self.wheel  # type: ignore
         assert self.source_dir, "Source directory isn't ready yet"
-        builder = WheelBuilder(self.source_dir, self.environment)
+        builder_cls = EditableBuilder if self.req.editable else WheelBuilder
+        builder = builder_cls(self.source_dir, self.environment)
         build_dir = self._get_wheel_dir()
         if not os.path.exists(build_dir):
             os.makedirs(build_dir)
@@ -348,10 +410,14 @@ class Candidate:
         elif self.req.editable:
             if self.environment.packages_path:
                 src_dir = self.environment.packages_path / "src"
-            elif os.getenv("VIRTUAL_ENV"):
-                src_dir = Path(os.environ["VIRTUAL_ENV"]) / "src"
             else:
-                src_dir = Path("src")
+                venv_prefix = get_venv_like_prefix(
+                    self.environment.interpreter.executable
+                )
+                if venv_prefix is not None:
+                    src_dir = venv_prefix / "src"
+                else:
+                    src_dir = Path("src")
             if not src_dir.is_dir():
                 src_dir.mkdir()
             ireq.ensure_has_source_dir(str(src_dir))
@@ -370,7 +436,7 @@ class Candidate:
         should_cache = False
         wheel_cache = self.environment.project.make_wheel_cache()
         assert self.link
-        if self.link.is_vcs:
+        if self.link.is_vcs and not self.req.editable:
             vcs = pip_shims.VcsSupport()
             vcs_backend = vcs.get_backend_for_scheme(self.link.scheme)
             if vcs_backend and vcs_backend.is_immutable_rev_checkout(
