@@ -3,9 +3,9 @@ import os
 from pathlib import Path
 from typing import Any, Callable, Dict, Iterator, MutableMapping, Optional, TypeVar
 
-import appdirs
-import atoml
 import click
+import platformdirs
+import tomlkit
 
 from pdm import termui
 from pdm.exceptions import NoConfigError
@@ -33,7 +33,7 @@ def load_config(file_path: Path) -> Dict[str, Any]:
 
     if not file_path.is_file():
         return {}
-    return get_item(dict(atoml.parse(file_path.read_text("utf-8"))))  # type: ignore
+    return get_item(dict(tomlkit.parse(file_path.read_text("utf-8"))))  # type: ignore
 
 
 def ensure_boolean(val: Any) -> bool:
@@ -74,24 +74,46 @@ class Config(MutableMapping[str, str]):
     pypi_url, verify_ssl = get_pypi_source()
     _config_map: Dict[str, ConfigItem] = {
         "cache_dir": ConfigItem(
-            "The root directory of cached files", appdirs.user_cache_dir("pdm"), True
+            "The root directory of cached files",
+            platformdirs.user_cache_dir("pdm"),
+            True,
+        ),
+        "check_update": ConfigItem(
+            "Check if there is any newer version available",
+            True,
+            True,
+            coerce=ensure_boolean,
         ),
         "auto_global": ConfigItem(
-            "Use global package implicity if no local project is found",
+            "Use global package implicitly if no local project is found",
             False,
             True,
             "PDM_AUTO_GLOBAL",
             coerce=ensure_boolean,
         ),
+        "build_isolation": ConfigItem(
+            "Isolate build environment from the project environment",
+            True,
+            False,
+            "PDM_BUILD_ISOLATION",
+            ensure_boolean,
+        ),
+        "project_max_depth": ConfigItem(
+            "The max depth to search for a project through the parents",
+            5,
+            True,
+            env_var="PDM_PROJECT_MAX_DEPTH",
+            coerce=int,
+        ),
         "strategy.update": ConfigItem(
             "The default strategy for updating packages", "reuse", False
         ),
         "strategy.save": ConfigItem(
-            "Specify how to save versions when a package is added", "compatible", False
+            "Specify how to save versions when a package is added", "minimum", False
         ),
         "strategy.resolve_max_rounds": ConfigItem(
             "Specify the max rounds of resolution process",
-            1000,
+            10000,
             env_var="PDM_RESOLVE_MAX_ROUDNS",
             coerce=int,
         ),
@@ -130,6 +152,10 @@ class Config(MutableMapping[str, str]):
             False,
             coerce=ensure_boolean,
         ),
+        "feature.install_cache_method": ConfigItem(
+            "`symlink` or `pth` to create links to the cached installation",
+            "symlink",
+        ),
     }
     del pypi_url, verify_ssl
 
@@ -166,7 +192,7 @@ class Config(MutableMapping[str, str]):
             temp[last] = value
 
         with self._config_file.open("w", encoding="utf-8") as fp:
-            atoml.dump(toml_data, fp)  # type: ignore
+            tomlkit.dump(toml_data, fp)  # type: ignore
 
     def __getitem__(self, key: str) -> Any:
         env_var = self._config_map[key].env_var
