@@ -10,7 +10,7 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, Iterable, Type, cast
 from urllib.parse import urlparse
 
-import atoml
+import tomlkit
 from pythonfinder import Finder
 from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
 
@@ -35,7 +35,7 @@ from pdm.utils import (
     find_project_root,
     find_python_in_path,
     get_in_project_venv_python,
-    is_venv_python,
+    get_venv_like_prefix,
 )
 
 if TYPE_CHECKING:
@@ -64,7 +64,11 @@ class Project:
         self.core = core
 
         if root_path is None:
-            root_path = find_project_root() if not is_global else self.GLOBAL_PROJECT
+            root_path = (
+                find_project_root(max_depth=self.global_config["project_max_depth"])
+                if not is_global
+                else self.GLOBAL_PROJECT
+            )
         if not is_global and root_path is None and self.global_config["auto_global"]:
             self.core.ui.echo(
                 "Project is not found, fallback to the global project",
@@ -92,7 +96,7 @@ class Project:
     @property
     def pyproject(self) -> dict | None:
         if not self._pyproject and self.pyproject_file.exists():
-            data = atoml.parse(self.pyproject_file.read_text("utf-8"))
+            data = tomlkit.parse(self.pyproject_file.read_text("utf-8"))
             self._pyproject = cast(dict, data)
         return self._pyproject
 
@@ -112,7 +116,7 @@ class Project:
         if not self._lockfile:
             if not self.lockfile_file.is_file():
                 raise ProjectError("Lock file does not exist.")
-            data = atoml.parse(self.lockfile_file.read_text("utf-8"))
+            data = tomlkit.parse(self.lockfile_file.read_text("utf-8"))
             self._lockfile = cast(dict, data)
         return self._lockfile
 
@@ -134,7 +138,7 @@ class Project:
     @cached_property
     def global_config(self) -> Config:
         """Read-and-writable configuration dict for global settings"""
-        return Config(Path.home() / ".pdm" / "config.toml", is_global=True)
+        return Config(self.GLOBAL_PROJECT.with_name("config.toml"), is_global=True)
 
     @cached_property
     def project_config(self) -> Config:
@@ -174,7 +178,9 @@ class Project:
         else:
             suffix = ""
             scripts = "bin"
-        virtual_env = os.getenv("VIRTUAL_ENV")
+
+        # Resolve virtual environments from env-vars
+        virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
         if config["use_venv"] and virtual_env:
             return PythonInfo.from_path(
                 os.path.join(virtual_env, scripts, f"python{suffix}")
@@ -199,7 +205,7 @@ class Project:
             # compatible with the exact version
             env.python_requires = PySpecSet(f"=={self.python.version}")
             return env
-        if self.config["use_venv"] and is_venv_python(self.python.executable):
+        if self.config["use_venv"] and get_venv_like_prefix(self.python.executable):
             # Only recognize venv created by python -m venv and virtualenv>20
             return GlobalEnvironment(self)
         return Environment(self)
@@ -303,6 +309,7 @@ class Project:
                 url=expand_env_vars_in_auth(s["url"]),
                 verify_ssl=s.get("verify_ssl", True),
                 name=s.get("name", urlparse(s["url"]).hostname),
+                type=s.get("type", "index"),
             )
             for s in sources
         ]
@@ -347,8 +354,8 @@ class Project:
         )
 
         repository = self.get_repository(cls=self.core.repository_class)
-        requires_python = self.environment.python_requires
         allow_prereleases = self.allow_prereleases
+        overrides = self.tool_settings.get("overrides", {})
         if strategy != "all" and not self.is_lockfile_compatible():
             self.core.ui.echo(
                 "Updating the whole lock file as it is not compatible with PDM",
@@ -357,11 +364,11 @@ class Project:
             )
             strategy = "all"
         if not for_install and strategy == "all":
-            return BaseProvider(repository, requires_python, allow_prereleases)
+            return BaseProvider(repository, allow_prereleases, overrides)
 
         locked_repository = self.locked_repository
         if for_install:
-            return BaseProvider(locked_repository, requires_python, allow_prereleases)
+            return BaseProvider(locked_repository, allow_prereleases, overrides)
         provider_class = (
             ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
         )
@@ -369,8 +376,8 @@ class Project:
             locked_repository.all_candidates,
             tracked_names or (),
             repository,
-            requires_python,
             allow_prereleases,
+            overrides,
         )
 
     def get_reporter(
@@ -391,7 +398,7 @@ class Project:
         return SpinnerReporter(spinner or termui.DummySpinner(), requirements)
 
     def get_lock_metadata(self) -> dict[str, Any]:
-        content_hash = atoml.string("sha256:" + self.get_content_hash("sha256"))
+        content_hash = tomlkit.string("sha256:" + self.get_content_hash("sha256"))
         content_hash.trivia.trail = "\n\n"
         return {"lock_version": self.LOCKFILE_VERSION, "content_hash": content_hash}
 
@@ -402,7 +409,7 @@ class Project:
 
         if write:
             with atomic_open_for_write(self.lockfile_file) as fp:
-                atoml.dump(toml_data, fp)  # type: ignore
+                tomlkit.dump(toml_data, fp)  # type: ignore
             if show_message:
                 self.core.ui.echo(f"Changes are written to {termui.green('pdm.lock')}.")
             self._lockfile = None
@@ -425,6 +432,7 @@ class Project:
             "dev-dependencies": self.tool_settings.get("dev-dependencies", {}),
             "optional-dependencies": self.meta.get("optional-dependencies", {}),
             "requires-python": self.meta.get("requires-python", ""),
+            "overrides": self.tool_settings.get("overrides", {}),
         }
         pyproject_content = json.dumps(dump_data, sort_keys=True)
         hasher = hashlib.new(algo)
@@ -495,7 +503,7 @@ class Project:
         with atomic_open_for_write(
             self.pyproject_file.as_posix(), encoding="utf-8"
         ) as f:
-            atoml.dump(self.pyproject, f)  # type: ignore
+            tomlkit.dump(self.pyproject, f)  # type: ignore
         if show_message:
             self.core.ui.echo(
                 f"Changes are written to {termui.green('pyproject.toml')}."
@@ -505,9 +513,45 @@ class Project:
     @property
     def meta(self) -> Metadata:
         if not self.pyproject:
-            self.pyproject = {"project": atoml.table()}
-        m = Metadata(self.pyproject_file, self.pyproject.get("project", {}))
+            self.pyproject = {"project": tomlkit.table()}
+        project_meta = self.pyproject["project"]
+        meta_version = project_meta.get("version")
+        updated = False
+        # Move version to tool table
+        if isinstance(meta_version, dict):
+            if "version" in self.tool_settings:
+                self.core.ui.echo(
+                    "WARNING: Removing dynamic `version` from [project] table",
+                    fg="yellow",
+                    err=True,
+                )
+            else:
+                self.core.ui.echo(
+                    "WARNING: Moving dynamic `version` from [project] to [tool.pdm]",
+                    fg="yellow",
+                    err=True,
+                )
+                self.tool_settings["version"] = meta_version
+            del project_meta["version"]
+            updated = True
+        # Delete classifiers from dynamic
+        dynamic_fields = project_meta.get("dynamic", [])
+        if "classifiers" in dynamic_fields:
+            self.core.ui.echo(
+                "WARNING: Dynamic `classifiers` is no longer supported, "
+                "please supply all classifiers manually",
+                fg="yellow",
+                err=True,
+            )
+            dynamic_fields.remove("classifiers")
+            if not dynamic_fields:
+                del project_meta["dynamic"]
+            updated = True
+        if updated:
+            self.write_pyproject(False)
+        m = Metadata(self.pyproject_file, False)
         m._metadata = self.pyproject.get("project", {})
+        m._tool_settings = self.tool_settings
         return m
 
     def init_global_project(self) -> None:
@@ -562,10 +606,12 @@ dependencies = ["pip", "setuptools", "wheel"]
         if not python_spec:
             if config.get("python.use_pyenv", True) and PYENV_INSTALLED:
                 pyenv_shim = os.path.join(PYENV_ROOT, "shims", "python3")
+                if os.name == "nt":
+                    pyenv_shim += ".bat"
                 if os.path.exists(pyenv_shim):
                     yield PythonInfo.from_path(pyenv_shim)
-                elif os.path.exists(pyenv_shim[:-1]):
-                    yield PythonInfo.from_path(pyenv_shim[:-1])
+                elif os.path.exists(pyenv_shim.replace("python3", "python")):
+                    yield PythonInfo.from_path(pyenv_shim.replace("python3", "python"))
             if config.get("use_venv"):
                 python = get_in_project_venv_python(self.root)
                 if python:
