@@ -1,44 +1,55 @@
 from __future__ import annotations
 
-from typing import Any, Iterable, Iterator, Mapping
+from typing import TYPE_CHECKING
 
 from resolvelib import AbstractProvider
-from resolvelib.resolvers import RequirementInformation
 
 from pdm.models.candidates import Candidate
-from pdm.models.repositories import BaseRepository
-from pdm.models.requirements import Requirement
-from pdm.models.specifiers import PySpecSet
-from pdm.utils import url_without_fragments
+from pdm.models.requirements import parse_requirement
+from pdm.resolver.python import (
+    PythonCandidate,
+    PythonRequirement,
+    find_python_matches,
+    is_python_satisfied_by,
+)
+from pdm.utils import is_url, url_without_fragments
+
+if TYPE_CHECKING:
+    from typing import Any, Iterable, Iterator, Mapping, Sequence
+
+    from resolvelib.resolvers import RequirementInformation
+
+    from pdm._types import Comparable
+    from pdm.models.repositories import BaseRepository
+    from pdm.models.requirements import Requirement
 
 
 class BaseProvider(AbstractProvider):
     def __init__(
         self,
         repository: BaseRepository,
-        requires_python: PySpecSet,
         allow_prereleases: bool | None = None,
+        overrides: dict[str, str] = None,
     ) -> None:
         self.repository = repository
-        self.requires_python = requires_python  # Root python_requires value
         self.allow_prereleases = allow_prereleases  # Root allow_prereleases value
-        self.requires_python_collection: dict[str, PySpecSet] = {}
-        self.summary_collection: dict[str, str] = {}
         self.fetched_dependencies: dict[str, list[Requirement]] = {}
+        self.overrides = overrides or {}
+        self._known_depth: dict[str, int] = {}
 
     def requirement_preference(self, requirement: Requirement) -> tuple:
         """Return the preference of a requirement to find candidates.
 
         - Editable requirements are preferered.
         - File links are preferred.
-        - The one with narrower specifierset is perferred.
+        - The one with narrower specifierset is preferred.
         """
         editable = requirement.editable
         is_file = requirement.is_file_or_url
         is_prerelease = (
-            bool(requirement.specifier.prereleases)
-            if requirement.specifier is not None
-            else False
+            requirement.prerelease
+            or requirement.specifier is not None
+            and bool(requirement.specifier.prereleases)
         )
         specifier_parts = len(requirement.specifier) if requirement.specifier else 0
         return (editable, is_file, is_prerelease, specifier_parts)
@@ -52,8 +63,55 @@ class BaseProvider(AbstractProvider):
         resolutions: dict[str, Candidate],
         candidates: dict[str, Iterator[Candidate]],
         information: dict[str, Iterator[RequirementInformation]],
-    ) -> int:
-        return sum(1 for _ in candidates[identifier])
+        backtrack_causes: Sequence[RequirementInformation],
+    ) -> Comparable:
+        is_top = any(parent is None for _, parent in information[identifier])
+        is_backtrack_cause = any(
+            requirement.identify() == identifier
+            or parent
+            and parent.identify() == identifier
+            for requirement, parent in backtrack_causes
+        )
+        if is_top:
+            dep_depth = 1
+        else:
+            parent_depths = (
+                self._known_depth[parent.identify()] if parent is not None else 0
+                for _, parent in information[identifier]
+            )
+            dep_depth = min(parent_depths) + 1
+        self._known_depth[identifier] = dep_depth
+        is_file_or_url = any(
+            not requirement.is_named for requirement, _ in information[identifier]
+        )
+        operators = [
+            spec.operator
+            for req, _ in information[identifier]
+            if req.specifier is not None
+            for spec in req.specifier
+        ]
+        is_python = identifier == "python"
+        is_pinned = any(op[:2] == "==" for op in operators)
+        is_free = bool(operators)
+        return (
+            not is_python,
+            not is_top,
+            not is_file_or_url,
+            not is_pinned,
+            not is_backtrack_cause,
+            dep_depth,
+            is_free,
+            identifier,
+        )
+
+    def get_override_candidates(self, identifier: str) -> Iterable[Candidate]:
+        requested = self.overrides[identifier]
+        if is_url(requested):
+            requested = f"{identifier} @ {requested}"
+        else:
+            requested = f"{identifier}=={requested}"
+        req = parse_requirement(requested)
+        return self.repository.find_candidates(req, self.allow_prereleases)
 
     def find_matches(
         self,
@@ -61,49 +119,54 @@ class BaseProvider(AbstractProvider):
         requirements: Mapping[str, Iterator[Requirement]],
         incompatibilities: Mapping[str, Iterator[Candidate]],
     ) -> Iterable[Candidate]:
+        incompat = list(incompatibilities[identifier])
+        if identifier == "python":
+            candidates = find_python_matches(
+                identifier, requirements, self.repository.environment
+            )
+            return [c for c in candidates if c not in incompat]
+        elif identifier in self.overrides:
+            return self.get_override_candidates(identifier)
         reqs = sorted(
             requirements[identifier], key=self.requirement_preference, reverse=True
         )
         file_req = next((req for req in reqs if not req.is_named), None)
-        incompat = list(incompatibilities[identifier])
         if file_req:
             can = Candidate(file_req, self.repository.environment)
             can.metadata
             candidates = [can]
         else:
+            req = reqs[0]
             candidates = self.repository.find_candidates(
-                reqs[0],
-                self.requires_python,
-                self.allow_prereleases,
+                req, req.prerelease or self.allow_prereleases
             )
         return [
             can
             for can in candidates
-            if all(self.is_satisfied_by(r, can) for r in reqs) and can not in incompat
+            if can not in incompat and all(self.is_satisfied_by(r, can) for r in reqs)
         ]
 
     def is_satisfied_by(self, requirement: Requirement, candidate: Candidate) -> bool:
+        if isinstance(requirement, PythonRequirement):
+            return is_python_satisfied_by(requirement, candidate)
+        elif candidate.identify() in self.overrides:
+            return True
         if not requirement.is_named:
             return not candidate.req.is_named and url_without_fragments(
                 candidate.req.url
             ) == url_without_fragments(requirement.url)
-        if not candidate.version:
-            candidate.metadata
-        if getattr(candidate, "_preferred", False) and not candidate._requires_python:
-            candidate.requires_python = str(
-                self.repository.get_dependencies(candidate)[1]
-            )
-        allow_prereleases = self.allow_prereleases
-        if allow_prereleases is None:
-            # if not specified, should allow what `find_candidates()` returns
-            allow_prereleases = True
-        requires_python = self.requires_python & requirement.requires_python
-        return requirement.specifier.contains(
-            candidate.version, allow_prereleases
-        ) and requires_python.is_subset(candidate.requires_python)
+        version = candidate.version or candidate.metadata.version
+        # Allow prereleases if: 1) it is not specified in the tool settings or
+        # 2) the candidate doesn't come from PyPI index.
+        allow_prereleases = (
+            self.allow_prereleases in (True, None) or not candidate.req.is_named
+        )
+        return requirement.specifier.contains(version, allow_prereleases)
 
     def get_dependencies(self, candidate: Candidate) -> list[Requirement]:
-        deps, requires_python, summary = self.repository.get_dependencies(candidate)
+        if isinstance(candidate, PythonCandidate):
+            return []
+        deps, requires_python, _ = self.repository.get_dependencies(candidate)
 
         # Filter out incompatible dependencies(e.g. functools32) early so that
         # we don't get errors when building wheels.
@@ -112,22 +175,27 @@ class BaseProvider(AbstractProvider):
             if (
                 dep.requires_python
                 & requires_python
-                & self.requires_python
                 & candidate.req.requires_python
+                & self.repository.environment.python_requires
             ).is_impossible:
                 continue
             dep.requires_python &= candidate.req.requires_python
             valid_deps.append(dep)
-
         candidate_key = self.identify(candidate)
-        self.fetched_dependencies[candidate_key] = valid_deps
-        self.summary_collection[candidate.req.key] = summary
-        self.requires_python_collection[candidate.req.key] = requires_python
+        self.fetched_dependencies[candidate_key] = valid_deps[:]
+        # A candidate contributes to the Python requirements only when:
+        # It isn't an optional dependency, or the requires-python doesn't cover
+        # the req's requires-python.
+        # For example, A v1 requires python>=3.6, it not eligible on a project with
+        # requires-python=">=2.7". But it is eligible if A has environment marker
+        # A1; python_version>='3.8'
+        new_requires_python = (
+            candidate.req.requires_python & self.repository.environment.python_requires
+        )
+        if not requires_python.is_superset(new_requires_python):
+            valid_deps.append(PythonRequirement.from_pyspec_set(requires_python))
         return valid_deps
 
-    def get_hashes(self, candidate: Candidate) -> dict[str, str] | None:
-        return self.repository.get_hashes(candidate)
-
 
 class ReusePinProvider(BaseProvider):
     """A provider that reuses preferred pins if possible.
@@ -140,9 +208,10 @@ class ReusePinProvider(BaseProvider):
         self,
         preferred_pins: dict[str, Candidate],
         tracked_names: Iterable[str],
-        *args: Any
+        *args: Any,
+        **kwargs: Any,
     ) -> None:
-        super().__init__(*args)
+        super().__init__(*args, **kwargs)
         self.preferred_pins = preferred_pins
         self.tracked_names = set(tracked_names)
 
@@ -200,8 +269,10 @@ class EagerUpdateProvider(ReusePinProvider):
         resolutions: dict[str, Candidate],
         candidates: dict[str, Iterator[Candidate]],
         information: dict[str, Iterator[RequirementInformation]],
-    ) -> int:
+        backtrack_causes: Sequence[RequirementInformation],
+    ) -> Comparable:
         # Resolve tracking packages so we have a chance to unpin them first.
-        if identifier in self.tracked_names:
-            return -1
-        return super().get_preference(identifier, resolutions, candidates, information)
+        (python, *others) = super().get_preference(
+            identifier, resolutions, candidates, information, backtrack_causes
+        )
+        return (python, identifier not in self.tracked_names, *others)
