@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import atexit
 import contextlib
 import functools
@@ -7,12 +9,12 @@ import os
 import sys
 from itertools import zip_longest
 from tempfile import mktemp
-from typing import List, Optional
+from typing import Any, Callable, Iterator, List, Optional, Sequence, Union
 
 import click
 from click._compat import strip_ansi
 
-from pdm._vendor import halo
+from pdm._vendor import colorama, halo
 from pdm._vendor.log_symbols.symbols import is_supported as supports_unicode
 
 logger = logging.getLogger(__name__)
@@ -38,14 +40,18 @@ def centerize(text: str, length: int) -> str:
 
 
 def supports_ansi() -> bool:
-    """Check if the current environment supports ANSI colors"""
-    if os.getenv("CI"):
-        return False
-    stream = sys.stdout
-    if not hasattr(stream, "fileno"):
+    if os.getenv("CI") or not hasattr(sys.stdout, "fileno"):
         return False
+    if sys.platform == "win32":
+        return (
+            os.getenv("ANSICON") is not None
+            or os.getenv("WT_SESSION") is not None
+            or "ON" == os.getenv("ConEmuANSI")
+            or "xterm" == os.getenv("Term")
+        )
+
     try:
-        return os.isatty(stream.fileno())
+        return os.isatty(sys.stdout.fileno())
     except io.UnsupportedOperation:
         return False
 
@@ -69,17 +75,20 @@ class DummySpinner:
     But only display text onto screen.
     """
 
-    def start(self, text: str):
+    def start(self, text: str) -> None:
         click.echo(text)
 
-    succeed = fail = stop_and_persist = start
+    def stop_and_persist(self, symbol: str = " ", text: Optional[str] = None) -> None:
+        click.echo(symbol + " " + (text or ""))
+
+    succeed = fail = start
 
     text = property(lambda self: "", start)
 
-    def __enter__(self):
+    def __enter__(self) -> DummySpinner:
         return self
 
-    def __exit__(self, *args):
+    def __exit__(self, *args: Any) -> None:
         pass
 
 
@@ -90,6 +99,10 @@ class UI:
         self.verbosity = verbosity
         self._indent = ""
         self.supports_ansi = not no_ansi if no_ansi is not None else supports_ansi()
+        if not self.supports_ansi:
+            colorama.init()
+        else:
+            colorama.deinit()
 
     def set_verbosity(self, verbosity: int) -> None:
         self.verbosity = verbosity
@@ -99,7 +112,7 @@ class UI:
         message: str = "",
         err: bool = False,
         verbosity: int = NORMAL,
-        **kwargs,
+        **kwargs: Any,
     ) -> None:
         if self.verbosity >= verbosity:
             click.secho(
@@ -107,7 +120,7 @@ class UI:
             )
 
     def display_columns(
-        self, rows: List[List[str]], header: Optional[List[str]] = None
+        self, rows: Sequence[Sequence[str]], header: Optional[List[str]] = None
     ) -> None:
         """Print rows in aligned columns.
 
@@ -115,7 +128,7 @@ class UI:
         :param header: a list of header strings.
         """
 
-        def get_aligner(align):
+        def get_aligner(align: str) -> Callable:
             if align == ">":
                 return rjust
             if align == "^":
@@ -141,7 +154,7 @@ class UI:
                 " ".join(
                     aligner(head, size)
                     for aligner, head, size in zip(aligners, header, sizes)
-                )
+                ).rstrip()
             )
             # Print a separator
             self.echo(" ".join("-" * size for size in sizes))
@@ -150,11 +163,11 @@ class UI:
                 " ".join(
                     aligner(item, size)
                     for aligner, item, size in zip(aligners, row, sizes)
-                )
+                ).rstrip()
             )
 
     @contextlib.contextmanager
-    def indent(self, prefix):
+    def indent(self, prefix: str) -> Iterator[None]:
         """Indent the following lines with a prefix."""
         _indent = self._indent
         self._indent += prefix
@@ -162,7 +175,7 @@ class UI:
         self._indent = _indent
 
     @contextlib.contextmanager
-    def logging(self, type_: str = "install"):
+    def logging(self, type_: str = "install") -> Iterator[logging.Logger]:
         """A context manager that opens a file for logging when verbosity is NORMAL or
         print to the stdout otherwise.
         """
@@ -177,7 +190,7 @@ class UI:
         pip_logger = logging.getLogger("pip.subprocessor")
         pip_logger.handlers[:] = [handler]
 
-        def cleanup():
+        def cleanup() -> None:
             try:
                 os.unlink(file_name)
             except OSError:
@@ -188,7 +201,7 @@ class UI:
         except Exception:
             if self.verbosity < DETAIL:
                 logger.exception("Error occurs")
-                self.echo(yellow(f"See {file_name} for detailed debug log."))
+                self.echo(yellow(f"See {file_name} for detailed debug log."), err=True)
             raise
         else:
             atexit.register(cleanup)
@@ -196,18 +209,22 @@ class UI:
             logger.handlers.remove(handler)
             pip_logger.handlers.remove(handler)
 
-    def open_spinner(self, title: str, spinner: str = "dots"):
+    def open_spinner(
+        self, title: str, spinner: str = "dots"
+    ) -> Union[DummySpinner, halo.Halo]:
         """Open a spinner as a context manager."""
         if self.verbosity >= DETAIL or not self.supports_ansi:
             return DummySpinner()
         else:
-            return halo.Halo(title, spinner=spinner, indent=self._indent)
+            return halo.Halo(  # type: ignore
+                title, spinner=spinner, indent=self._indent
+            )
 
 
 class Emoji:
     """A collection of emoji characters used in terminal output"""
 
-    if supports_unicode():
+    if supports_unicode():  # type: ignore
         SUCC = "🎉"
         LOCK = "🔒"
     else:
