@@ -427,20 +427,37 @@ def path_replace(pattern: str, replace_with: str, dest: str) -> str:
     )
 
 
+def is_path_relative_to(path: str | Path, other: str | Path) -> bool:
+    try:
+        Path(path).relative_to(other)
+    except ValueError:
+        return False
+    return True
+
+
 def is_venv_python(interpreter: str | Path) -> bool:
     """Check if the given interpreter path is from a virtualenv"""
     interpreter = Path(interpreter)
     if interpreter.parent.parent.joinpath("pyvenv.cfg").exists():
         return True
+
     virtual_env = os.getenv("VIRTUAL_ENV")
-    if virtual_env:
-        try:
-            interpreter.relative_to(virtual_env)
-        except ValueError:
-            pass
-        else:
-            return True
-    return False
+    return bool(virtual_env and is_path_relative_to(interpreter, virtual_env))
+
+
+def get_venv_like_prefix(interpreter: str | Path) -> Path | None:
+    """Check if the given interpreter path is from a virtualenv,
+    and return the prefix if found.
+    """
+    interpreter = Path(interpreter)
+    prefix = interpreter.parent.parent
+    if prefix.joinpath("pyvenv.cfg").exists():
+        return prefix
+
+    virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
+    if virtual_env and is_path_relative_to(interpreter, virtual_env):
+        return Path(virtual_env)
+    return None
 
 
 def find_python_in_path(path: str | Path) -> Path | None:
@@ -498,17 +515,39 @@ def is_editable(dist: Distribution) -> bool:
 
 def pdm_scheme(base: str) -> dict[str, str]:
     """Return a PEP 582 style install scheme"""
-    if "pdm" not in sysconfig._INSTALL_SCHEMES:  # type: ignore
+    if "pep582" not in sysconfig._INSTALL_SCHEMES:  # type: ignore
         bin_prefix = "Scripts" if os.name == "nt" else "bin"
-        sysconfig._INSTALL_SCHEMES["pdm"] = {  # type: ignore
-            "stdlib": "{base}/lib",
-            "platstdlib": "{base}/lib",
-            "purelib": "{base}/lib",
-            "platlib": "{base}/lib",
-            "include": "{base}/include",
-            "scripts": "{base}/%s" % bin_prefix,
-            "data": "{base}",
-            "prefix": "{base}",
-            "headers": "{base}/include",
+        sysconfig._INSTALL_SCHEMES["pep582"] = {  # type: ignore
+            "stdlib": "{pep582_base}/lib",
+            "platstdlib": "{pep582_base}/lib",
+            "purelib": "{pep582_base}/lib",
+            "platlib": "{pep582_base}/lib",
+            "include": "{pep582_base}/include",
+            "scripts": "{pep582_base}/%s" % bin_prefix,
+            "data": "{pep582_base}",
+            "prefix": "{pep582_base}",
+            "headers": "{pep582_base}/include",
         }
-    return sysconfig.get_paths("pdm", vars={"base": base}, expand=True)
+    return sysconfig.get_paths("pep582", vars={"pep582_base": base}, expand=True)
+
+
+def is_url(url: str) -> bool:
+    """Check if the given string is a URL"""
+    return bool(parse.urlparse(url).scheme)
+
+
+@functools.lru_cache()
+def fs_supports_symlink() -> bool:
+    if not hasattr(os, "symlink"):
+        return False
+    if sys.platform == "win32":
+        with tempfile.NamedTemporaryFile(prefix="TmP") as tmp_file:
+            temp_dir = os.path.dirname(tmp_file.name)
+            dest = os.path.join(temp_dir, "{}-{}".format(tmp_file.name, "b"))
+            try:
+                os.symlink(tmp_file.name, dest)
+                return True
+            except (OSError, NotImplementedError):
+                return False
+    else:
+        return True
