@@ -211,7 +211,10 @@ def test_vcs_candidate_in_subdirectory(project, is_editable):
     )
     req = parse_requirement(line, is_editable)
     candidate = Candidate(req, project.environment)
-    assert candidate.get_dependencies_from_metadata() == ["django"]
+    expected_deps = ["django"]
+    if is_editable:
+        expected_deps.append("editables")
+    assert candidate.get_dependencies_from_metadata() == expected_deps
     assert candidate.version == "0.1.0"
 
 
